@@ -660,11 +660,18 @@ frdp_post_connect (freerdp *freerdp_session)
 static void
 frdp_post_disconnect (freerdp *instance)
 {
-  rdpContext *context;
+  FrdpSession *self;
+  rdpContext  *context;
 
   if (!instance || !instance->context)
     return;
 
+  self = ((frdpContext *) instance->context)->self;
+
+  g_signal_handlers_disconnect_by_func (self->priv->display, G_CALLBACK (frdp_session_draw), self);
+  g_signal_handlers_disconnect_by_func (self->priv->display, G_CALLBACK (frdp_session_configure_event), self);
+  g_signal_handlers_disconnect_by_func (self->priv->display, G_CALLBACK (frdp_session_resize_supported_changed), self);
+
   context = instance->context;
   PubSub_UnsubscribeChannelConnected (context->pubSub,
                                       frdp_on_channel_connected_event_handler);
@@ -707,18 +714,28 @@ update (gpointer user_data)
 
   priv = self->priv;
 
+  if (freerdp_shall_disconnect (priv->freerdp_session)) {
+      priv->update_id = 0;
+      g_idle_add ((GSourceFunc) idle_close, self);
+
+      return FALSE;
+  }
+
   usedHandles = freerdp_get_event_handles (priv->freerdp_session->context,
                                            handles, ARRAYSIZE(handles));
   if (usedHandles == 0) {
       g_warning ("Failed to get FreeRDP event handle");
+      priv->update_id = 0;
       return FALSE;
   }
 
   status = WaitForMultipleObjects (usedHandles, handles, FALSE, SELECT_TIMEOUT);
   if (status == WAIT_TIMEOUT)
     return TRUE;
-  if (status == WAIT_FAILED)
+  if (status == WAIT_FAILED) {
+    priv->update_id = 0;
     return FALSE;
+  }
 
   if (!freerdp_check_event_handles (priv->freerdp_session->context)) {
     if (freerdp_get_last_error(priv->freerdp_session->context) == FREERDP_ERROR_SUCCESS) {
@@ -728,12 +745,6 @@ update (gpointer user_data)
     return TRUE;
   }
 
-  if (freerdp_shall_disconnect (priv->freerdp_session)) {
-      g_idle_add ((GSourceFunc) idle_close, self);
-
-      return FALSE;
-  }
-
   return TRUE;
 }
 
@@ -1088,19 +1099,19 @@ frdp_session_get_property (GObject    *object,
   switch (property_id)
     {
       case PROP_HOSTNAME:
-        g_value_set_string (value, settings->ServerHostname);
+        g_value_set_string (value, self->priv->hostname);
         break;
       case PROP_PORT:
-        g_value_set_uint (value, settings->ServerPort);
+        g_value_set_uint (value, self->priv->port);
         break;
       case PROP_USERNAME:
-        g_value_set_string (value, settings->Username);
+        g_value_set_string (value, self->priv->username);
         break;
       case PROP_PASSWORD:
-        g_value_set_string (value, settings->Password);
+        g_value_set_string (value, self->priv->password);
         break;
       case PROP_DOMAIN:
-        g_value_set_string (value, settings->Domain);
+        g_value_set_string (value, self->priv->domain);
         break;
       case PROP_DISPLAY:
         g_value_set_object (value, self->priv->display);
@@ -1304,7 +1315,19 @@ frdp_session_connect (FrdpSession         *self,
   self->priv->port = port;
 
   task = g_task_new (self, cancellable, callback, user_data);
-  g_task_run_in_thread (task, frdp_session_connect_thread);
+  /* Turn off the asynchronous connection via GTask thread as the FreeRDP
+   * process then runs in the new thread which makes it hard to cooperate
+   * with the original thread running UI leading to random race conditions.
+   * Turn it on again if there will be an async support for connection
+   * added to FreeRDP.
+   * The disadvantage is that the application freezes during connection
+   * for some time.
+
+     g_task_run_in_thread (task, frdp_session_connect_thread);
+
+   */
+
+  frdp_session_connect_thread (task, self, user_data, cancellable);
 
   g_object_unref (task);
 }
